Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func IsAllowedNetwork ¶ added in v0.14.27
IsAllowedNetwork returns true if the given host (IP or resolvable hostname) is in the set of allowed networks (CIDR format only).
Types ¶
type Connection ¶ added in v0.13.0
type Connection interface {
protocol.Connection
Type() string
Transport() string
RemoteAddr() net.Addr
Priority() int
String() string
Crypto() string
}
Connection is what we expose to the outside. It is a protocol.Connection that can be closed and has some metadata.
type ConnectionStatusEntry ¶ added in v1.2.0
type ListenerStatusEntry ¶ added in v1.2.0
type Model ¶
type Model interface {
protocol.Model
AddConnection(conn Connection, hello protocol.HelloResult)
Connection(remoteID protocol.DeviceID) (Connection, bool)
OnHello(protocol.DeviceID, net.Addr, protocol.HelloResult) error
GetHello(protocol.DeviceID) protocol.HelloIntf
}
type Service ¶ added in v0.13.0
type Service interface {
suture.Service
discover.AddressLister
ListenerStatus() map[string]ListenerStatusEntry
ConnectionStatus() map[string]ConnectionStatusEntry
NATType() string
}
Service listens and dials all configured unconnected devices, via supported dialers. Successful connections are handed to the model.
Source Files
¶
Directories
¶
| Path | Synopsis |
|---|---|
|
Registry tracks connections/addresses on which we are listening on, to allow us to pick a connection/address that has a NAT port mapping.
|
Registry tracks connections/addresses on which we are listening on, to allow us to pick a connection/address that has a NAT port mapping. |
Click to show internal directories.
Click to hide internal directories.